from flask_babel import lazy_gettext as _l
from wtforms import (
    FileField,
    HiddenField,
    IntegerField,
    PasswordField,
    RadioField,
    SelectField,
    StringField,
    TextAreaField,
)
from wtforms.fields.html5 import EmailField
from wtforms.validators import InputRequired
from wtforms.widgets.html5 import NumberInput

from CTFd.constants.config import (
    AccountVisibilityTypes,
    ChallengeVisibilityTypes,
    RegistrationVisibilityTypes,
    ScoreVisibilityTypes,
)
from CTFd.forms import BaseForm
from CTFd.forms.fields import SubmitField
from CTFd.utils.config import get_themes


class SetupForm(BaseForm):
    ctf_name = StringField(
        _l("Nom de l'événement"), description=_l("Le nom de votre événement PolyScav")
    )
    ctf_description = TextAreaField(
        _l("Description de l'événement"), description=_l("Description de la PolyScav")
    )
    user_mode = RadioField(
        _l("Mode utilisateur"),
        choices=[("teams", _l("Mode équipe")), ("users", _l("Mode utilisateur"))],
        default="teams",
        description=_l(
            "Contrôle si les utilisateurs se réunissent dans des équipes pour jouer (mode équipe) ou jouer en tant qu'elles (mode utilisateur)"
        ),
    )
    
    name = StringField(
        _l("Nom d'utilisateur admin"),
        description=_l("Votre nom d'utilisateur pour le compte d'administration"),
        validators=[InputRequired()],
    )
    email = EmailField(
        _l("Courriel administrateur"),
        description=_l("Votre adresse courriel pour le compte d'administration"),
        validators=[InputRequired()],
    )
    password = PasswordField(
        _l("Mot de passe administrateur"),
        description=_l("Votre mot de passe pour le compte d'administration"),
        validators=[InputRequired()],
    )
    admin_visible = SelectField(
        _l("Visibilité de l'administration"),
        description="Contrôlez si le compte administrateur participe aux défis et si le compte est visible",
        choices=[("true", "visible"), ("false", "Caché")],
        default="true",
    )

    ctf_logo = FileField(
        _l("Logo"),
        description=_l(
            "Logo à utiliser pour le site Web au lieu d'un nom. Utilisé comme bouton de page d'accueil. Facultatif."
        ),
    )
    ctf_banner = FileField(
        _l("Bannière"), description=_l("Bannière à utiliser pour la page d'accueil. Facultatif.")
    )
    ctf_small_icon = FileField(
        _l("Petite icône"),
        description=_l(
            "Favicon utilisé dans les navigateurs de l'utilisateur. Seuls les PNG ont accepté. Doit être 32x32px. Facultatif."
        ),
    )
    ctf_sponsord1 = FileField(
        _l("Logo sponsor1"),
        description=_l(
            "Favicon pour les organisateurs/sponsors de l'évènement. Seuls les PNG ont accepté. Doit être 100x100px. Facultatif."
        ),
    )
    ctf_sponsord2 = FileField(
        _l("Logo sponsor2"),
        description=_l(
            "Logo pour les organisateurs/sponsors de l'évènement. Seuls les PNG ont accepté. Doit être 100x100px. Facultatif."
        ),
    )
    ctf_theme = SelectField(
        _l("Thème"),
        description=_l("Thème polyscav à utiliser. Peut être modifié plus tard."),
        choices=list(zip(get_themes(), get_themes())),
        ## TODO: Replace back to DEFAULT_THEME (aka core) in CTFd 4.0
        default="core-beta",
    )
    theme_color = HiddenField(
        _l("Couleur du thème"),
        description=_l(
            "Couleur utilisée par le thème pour contrôler l'esthétique. Nécessite un support du thème. Facultatif."
        ),
    )

    verify_emails = SelectField(
        _l("Vérifiez les courriels"),
        description="Contrôlez si les utilisateurs doivent confirmer leurs adresses courriel avant de participer",
        choices=[("true", "Activé"), ("false", "Désactivé")],
        default="false",
    )
    team_size = IntegerField(
        widget=NumberInput(min=0),
        description="Quantité d'utilisateurs par équipe (mode équipe uniquement). Facultatif.",
    )
    team_creation = SelectField(
        _l("création d'équipe"),
        description="Contrôlez si les utilisateurs peuvent créer leurs propres équipes (mode équipe uniquement)",
        choices=[("true", "Activé"), ("false", "Désactivé")],
        default="false",
    )
    challenge_visibility = SelectField(
        "Visibilité des défis",
        description="Contrôlez si les utilisateurs doivent être connectés pour voir les défis",
        choices=[
            (ChallengeVisibilityTypes.PUBLIC, "Publique"),
            (ChallengeVisibilityTypes.PRIVATE, "Privée"),
            (ChallengeVisibilityTypes.ADMINS, "Admins uniquement"),
        ],
        default=ChallengeVisibilityTypes.PRIVATE,
    )
    account_visibility = SelectField(
        "Visibilité des comptes",
        description="Contrôlez si les comptes (utilisateurs et équipes) sont affichés à tout le monde, uniquement aux utilisateurs authentifiés, ou uniquement aux administrateurs",
        choices=[
            (AccountVisibilityTypes.PUBLIC, "Publique"),
            (AccountVisibilityTypes.PRIVATE, "Privée"),
            (AccountVisibilityTypes.ADMINS, "Admins uniquement"),
        ],
        default=AccountVisibilityTypes.PUBLIC,
    )
    score_visibility = SelectField(
        "Visibilité du score",
        description="Contrôlez si les résolutions/scores sont affichés au public, pour les utilisateurs connectés, cachés à tous les non-admins, ou seulement aux administrateurs",
        choices=[
            (ScoreVisibilityTypes.PUBLIC, "Publique"),
            (ScoreVisibilityTypes.PRIVATE, "Privée"),
            (ScoreVisibilityTypes.HIDDEN, "Caché"),
            (ScoreVisibilityTypes.ADMINS, "Admins uniquement"),
        ],
        default=AccountVisibilityTypes.PUBLIC,
    )
    registration_visibility = SelectField(
        "Visibilité de la création de compte",
        description="Contrôlez si la création de compte est activée pour tout le monde ou désactivée",
        choices=[
            (RegistrationVisibilityTypes.PUBLIC, "Publique"),
            (RegistrationVisibilityTypes.PRIVATE, "Privée"),
        ],
        default=RegistrationVisibilityTypes.PUBLIC,
    )

    start = StringField(
        _l("Heure de début"),
        description=_l("Heure où votre évènement devrait commencer."),
    )
    end = StringField(
        _l("Heure de fin"),
        description=_l("Heure où votre évènement devrait se terminer."),
    )
    submit = SubmitField(_l("Terminer"))
